%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  Plot ray paths calculated by calc_2d_raytrace.m
%%  Written by Jeff Shaman.
%%  Some minor changes, and all the errors: Eli Tziperman
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%  jj is the type of background calculation (Ks, BetaM, Um)
%%  The choices are 1) HA Mercator: haKs, BetaM, UbarM, u200
%%                  2) HK Mercator: hkKs, hkBetaM, hkU, u200
%%                  3) HA Not Mercator: haKs2, ha_dqdy,u200, u200
%%  k is the zonal wavenumber
%%  wv is the type of dx/dt calculation; 1 assumes the zonal wind
%%  and phase speed cancel; 2 includes the discrepancy in the zonal
%%  group velocity
%%  fr is the forcing region

close all; clear all

%% if not batch=1, plot each ray separately and pause after each
%% ray:
do_batch_run=1;

%% plot detailed diagnostics for each ray trace: useful if not
%% doing a batch run:
do_plot_ray_diagnostics=0;

%Eli:  added path to m_map routines:
addpath('/home/eli/matlab/m_map','-end');

load('Output/parameters.mat');
% number of locations from which trajectories were calculated:
Nlocations=length(frcy);
day=86400;

plot_end_time_step=Nsteps;
%% overrid locally: (comment out to plot the entire calculated paths:)
plot_end_time_step=min(72,Nsteps);


%% loop over all specified frequencies:
for iomega=1:length(frequencies)
  omega=frequencies(iomega);
  period=round((2*pi/omega)/day);

  %% loop over all specified wavenumbers:
  for ik=1:length(k_wavenumbers)
    k=k_wavenumbers(ik);

    %% loop over all specified initial locations of rays:
    for fr=1:Nlocations

      %% loop over all three root, although some may have not been
      %% calculated because they led to complex wavenumbers etc:
      for iroot=1:3
        fprintf(1,'wavenumber: %d, period: %d, forcing location: %d, root number: %d\n'...
                ,k,period,fr,iroot);
        %k=4;fr=4;iroot=1;
        %%Eli: moved input file to subdirectory:
        filename=sprintf('Output/raypath_k%d_period%d_location%d_root%d',k,period,fr,iroot);
        if exist(filename,'file')==0;
          disp(' ');
          disp(['*** input file does not exist: ',filename]);
          disp(' ');
        else
          a=load(filename);
          [m,n]=size(a);
          
          t=1:m;
          loc=a(:,17:18);
          trl=a(:,1:2);
          rnums=a(:,3:5);
          inums=a(:,6:8);
          rpchg=a(:,9:10);
          ipchg=a(:,11:12);
          wchg=a(:,13:16);
          rsom=a(:,19);
          isom=a(:,20);

          if do_plot_ray_diagnostics
            figure(1)
            subplot(3,2,1)
            plot(loc(:,2),loc(:,1))
            title('Ray Path')
            subplot(3,2,2)
            plot(t,rnums)
            legend('real l','real k','real Ks')
            subplot(3,2,3)
            plot(t,inums)
            legend('imag l','imag k','imag Ks')
            subplot(3,2,4)
            plot(t,wchg)
            legend('real dl/dt','real dk/dt','imag dl/dt','imag dk/dt')
            subplot(3,2,5)
            plot(t,rpchg,t,ipchg)
            legend('real dx/dt','real dy/dt','imag dx/dt','imag dy/dt')
          end
          
          figure(2)
          %% Eli: made some changes here:
          %% Deal with trajectories that leave the domain from
          %% east/ west sides and need to return from the other side:
          loc_length=length(loc(:,1));
          for i=1:length(loc(:,1)); 
            if loc(i,2)>147.5
              if i>1 then
                fprintf(1,['*** ray leaving domain: loc>147.5, shifting ' ...
                           'to loc-360. (i,loc)=(%d,%g).\n'],i,loc(i,2))
              end
              loc((i):loc_length,2)=loc((i):loc_length,2)-360;
              if i>1
                %% set to NaN to prevent plotting of ray path
                %% across this transition between east and west boundaries:
                loc(i,1)=NaN;
                loc(i,2)=NaN;
              end
            end
            if loc(i,2)<-212.5
              if i>1 then
                fprintf(1,['*** ray leaving domain: loc<-212.5, shifting ' ...
                           'to loc+360. (i,loc)=(%d,%g).\n'],i,loc(i,2))
              end
              loc((i):loc_length,2)=loc((i):loc_length,2)+360;
              if i>1
                %% set to NaN to prevent plotting of ray path
                %% across this transition between east and west boundaries:
                loc(i,1)=NaN;
                loc(i,2)=NaN;
              end
            end
          end
          subplot(2,1,1)
          m_proj('equidistant','lon',[-210 147.5],'lat',[-60 60]);
          m_plotarrow(loc(:,2),loc(:,1));
          m_coast('linewidth',2,'color','k');
          m_grid;
          if omega==0
            title(sprintf('Ray paths for k=%d, \\omega=%g'...
                          ,k,omega));
          else
            day=86400;
            title(sprintf('Ray paths for k=%d, 2\\pi/\\omega=%g days' ...
                          ,k,(2*pi/omega)/day));
          end
          
          %% Eli: I'd like all trajectories for a given (k,omega)
          %% combination to be on one map:
          if do_batch_run==1
            hold on
          end

          %% some plots that are relevant only to complex ray tracing:
          if do_complex_tracing==1
            subplot(2,1,2)
            r=6e6;
            inm(:,1)=ipchg(:,2).*rnums(:,1)*3600/r/r;
            inm(:,2)=rpchg(:,2).*inums(:,1)*3600/r/r;
            inm(:,3)=ipchg(:,1).*rnums(:,2)*3600/r/r;
            inm(:,4)=rpchg(:,1).*inums(:,2)*3600/r/r;
            
            Inm=sum(inm')';
            Inm2=[inm(:,2) inm(:,4)];
            Inm2=sum(Inm2')';
            plot(t,Inm,t,Inm2);
            title('Complex ray amplitudes something')
            hold on
            
            dcy=zeros(length(inm)+1,1);
            dcy2=dcy;dcy3=dcy;
            dcy(1)=1;dcy2(1)=1;dcy3(1)=1;dcy4=dcy*ones(1,3);
            for i=1:length(inm)
              dcy(i+1)=dcy(i)*exp(-1*(inm(i,1)+inm(i,2)+inm(i,3)+inm(i,4)));
              dcy2(i+1)=dcy2(i)*exp(-1*(abs(inm(i,1))+abs(inm(i,2)) ...
                                        +abs(inm(i,3))+abs(inm(i,4))));
              for j=1:4
                if inm(i,j)>0
                  inm2(i,j)=inm(i,j);
                else
                  inm2(i,j)=0;
                end
              end
              dcy3(i+1)=dcy3(i)*exp(-1*(inm2(i,1)+inm2(i,2)+inm2(i,3)+inm2(i,4)));
              dcy4(i+1,1)=dcy4(i,1)*exp(-1*Inm2(i));
            end
            
            tt=1:m+1;
            plot(tt(1:plot_end_time_step),1./dcy4(1:plot_end_time_step,1))
            title('Complex ray amplitudes something')
            fprintf(1,'start location=');
            fprintf(1,'%gN, %gE; ',loc(1,:)); 
            fprintf(1,'\n');
          end % end of plotting complex ray stuff on a lower panel
        
        end % end of if statment making sure input file exists

        %% Eli: to pause after each ray, uncomment this:
        if do_batch_run~=1
          pause
        end
        
      end % end of do loop over all 3 roots for initial l
    end % end of do loop over all start locations of rays.

    %% save figure:
    filename_fig=sprintf('Output/Figure_raypath_k%d_period%d',k,period);
    saveas(gcf,[filename_fig '.fig']);
    %saveas(gcf,[filename_fig '.pdf'],'pdf');
    save2pdf([filename_fig '.pdf'],gcf);
  
    %% with the following line, a single plot is produced for a
    %% given combination of (k,omega):
    close all

  end % end of do loop over wavenumbers

end % end of do loop over frequencies
